%
%
%  Find Spots Close To Surface for Imaris 7.3.0
%
%  Copyright Bitplane AG 2011
%
%
%  Installation:
%
%  - Copy this file into the XTensions folder in the Imaris installation directory
%  - You will find this function in the Image Processing menu
%
%    <CustomTools>
%      <SurpassTab>
%        <SurpassComponent name="bpSpots">
%          <Item name="Find Spots Close To Surface" icon="Matlab" tooltip="Find spots close to surface.">
%            <Command>MatlabXT::XTSpotsCloseToSurface(%i)</Command>
%          </Item>
%        </SurpassComponent>
%        <SurpassComponent name="bpSurfaces">
%          <Item name="Find Spots Close To Surface" icon="Matlab" tooltip="Find spots close to surface.">
%            <Command>MatlabXT::XTSpotsCloseToSurface(%i)</Command>
%          </Item>
%        </SurpassComponent>
%      </SurpassTab>
%	   <Menu>
%        <Item name="Find Spots Close To Surface" icon="Matlab" tooltip="Find spots close to surface.">
%          <Command>MatlabXT::XTSpotsCloseToSurface(%i)</Command>
%        </Item>
%	   </Menu>
%    </CustomTools>
% 
%
%  Description:
%   
%   Find the spots that are closer to a surface than a threshold, and 
%       create two new spots objects: One collects the spots close to the
%       surface, the other the farer. 
%   If multiple spots or surfaces are selected, each pair [spots, surface] 
%       is analyzed: The number of spots objects created is 2 times 
%       'number of spots' times 'number of surfaces'. 
%
%

function XTSpotsCloseToSurface(aImarisApplicationID)

% connect to Imaris interface
if ~isa(aImarisApplicationID, 'Imaris.IApplicationPrxHelper')
    javaaddpath ImarisLib.jar
    vImarisLib = ImarisLib;
    if ischar(aImarisApplicationID)
        aImarisApplicationID = round(str2double(aImarisApplicationID));
    end
    vImarisApplication = vImarisLib.GetApplication(aImarisApplicationID);
else
    vImarisApplication = aImarisApplicationID;
end

% the user has to create a scene with some spots and surface
vSurpassScene = vImarisApplication.GetSurpassScene;
if isequal(vSurpassScene, [])
    msgbox('Please create some Spots and Surface in the Surpass scene!')
    return
end

% get the spots and the surface object
vSpots = vImarisApplication.GetFactory.ToSpots(vImarisApplication.GetSurpassSelection);
vSurfaces = vImarisApplication.GetFactory.ToSurfaces(vImarisApplication.GetSurpassSelection);

vSpotsSelected = ~isequal(vSpots, []);
vSurfaceSelected = ~isequal(vSurfaces, []);
if vSpotsSelected
    vParent = vSpots.GetParent;
elseif vSurfaceSelected
    vParent = vSurfaces.GetParent;
else
    vParent = vSurpassScene;
end

% get the spots and surfaces
vSpotsSelection = 1;
vSurfaceSelection = 1;
vNumberOfSpots = 0;
vNumberOfSurfaces = 0;
vSpotsList = [];
vSurfacesList = [];
vSpotsName = {};
vSurfacesName = {};
for vIndex = 1:vParent.GetNumberOfChildren
    vItem = vParent.GetChild(vIndex-1);
    if vImarisApplication.GetFactory.IsSpots(vItem)
        vNumberOfSpots = vNumberOfSpots + 1;
        vSpotsList(vNumberOfSpots) = vIndex;
        vSpotsName{vNumberOfSpots} = char(vItem.GetName);
        
        if vSpotsSelected && isequal(vItem.GetName, vSpots.GetName)
            vSpotsSelection = vNumberOfSpots; 
        end
    elseif vImarisApplication.GetFactory.IsSurfaces(vItem)
        vNumberOfSurfaces = vNumberOfSurfaces + 1;
        vSurfacesList(vNumberOfSurfaces) = vIndex;
        vSurfacesName{vNumberOfSurfaces} = char(vItem.GetName);
        
        if vSurfaceSelected && isequal(vItem.GetName, vSurfaces.GetName)
            vSurfaceSelection = vNumberOfSurfaces;
        end
    end
end

if min(vNumberOfSpots,vNumberOfSurfaces) == 0
    msgbox('Please create some spots AND a surface object!')
    return
end

if vNumberOfSpots>1
    [vSpotsSelection,vOk] = listdlg('ListString',vSpotsName, ...
        'InitialValue', vSpotsSelection, 'SelectionMode','multiple', ...
        'ListSize',[300 300], 'Name','Find Spots Close To Surface', ...
        'PromptString',{'Please select the spots:'});
    if vOk<1, return, end
end
if vNumberOfSurfaces>1
    [vSurfaceSelection,vOk] = listdlg('ListString',vSurfacesName, ...
        'InitialValue', vSurfaceSelection, 'SelectionMode','multiple', ...
        'ListSize',[300 300], 'Name','Find Spots Close To Surface', ...
        'PromptString',{'Please select the surface:'});
    if vOk<1, return, end
end

vAnswer = inputdlg({'Please enter the threshold:'}, ...
    'Find Spots Close To Surface',1,{'5'});
if isempty(vAnswer), return, end
vThreshold = abs(str2double(vAnswer{1}));

vProgressDisplay = waitbar(0,'Finding Spots Close To Surface');

% compute the distances and create new spots objects
vNumberOfSurfacesSelected = numel(vSurfaceSelection);
vNumberOfSpotsSelected = numel(vSpotsSelection);

for vSurfaceIndex = 1:vNumberOfSurfacesSelected
    vItem = vParent.GetChild(vSurfacesList( ...
        vSurfaceSelection(vSurfaceIndex)) - 1);
    vSurface = vImarisApplication.GetFactory.ToSurfaces(vItem);

    vSurfaceVertices = [];
    for vIndex = 0:vSurface.GetNumberOfSurfaces - 1
      vSurfaceVertices = [vSurfaceVertices; vSurface.GetVertices(vIndex)];
    end
    vNumberOfVertices = size(vSurfaceVertices, 1);
    
    % limit the memory usage to 3*10000*vNumberOfSpots
    vLimit = 10000;
    if vNumberOfVertices > vLimit
        vIndices = round(linspace(1, vNumberOfVertices, vLimit));
        vSurfaceVertices = vSurfaceVertices(vIndices, :);
        vNumberOfVertices = vLimit;
    end
    
    vSparseVertices = 1:vNumberOfVertices;
    % sparse diagonals
    vSurfX = sparse(vSparseVertices, vSparseVertices, double(vSurfaceVertices(:,1)));
    vSurfY = sparse(vSparseVertices, vSparseVertices, double(vSurfaceVertices(:,2)));
    vSurfZ = sparse(vSparseVertices, vSparseVertices, double(vSurfaceVertices(:,3)));

    for vSpotsIndex = 1:vNumberOfSpotsSelected
        vItem = vParent.GetChild(vSpotsList( ...
            vSpotsSelection(vSpotsIndex)) - 1);
        vSpots = vImarisApplication.GetFactory.ToSpots(vItem);
        
		vSpotsPosition = vSpots.GetPositionsXYZ;
		vSpotsTime = vSpots.GetIndicesT;
		vSpotsRadius = vSpots.GetRadiiXYZ;
        vNumberOfSpots = size(vSpotsPosition, 1);
        
        % build the distance matrix
        vOnes = ones(vNumberOfVertices, vNumberOfSpots);
        vDistX = vOnes*diag(vSpotsPosition(:, 1)) - vSurfX*vOnes;
        vDistY = vOnes*diag(vSpotsPosition(:, 2)) - vSurfY*vOnes;
        vDistZ = vOnes*diag(vSpotsPosition(:, 3)) - vSurfZ*vOnes;
        
        vDistances = sqrt( min( vDistX.^2+vDistY.^2+vDistZ.^2, [], 1 ) );
        
        vSpots.SetVisible(false);
        
        vSpotsClose = vDistances <= vThreshold;
        if any(vSpotsClose)
          vNewSpotsClose = vImarisApplication.GetFactory.CreateSpots;
		  % The radius can be setted to zero because it is overwritten on the next line
		  % Only an array with the right size is needed
          vNewSpotsClose.Set(vSpotsPosition(vSpotsClose, :), ...
              vSpotsTime(vSpotsClose), zeros(sum(vSpotsClose),1));
          vNewSpotsClose.SetRadiiXYZ(vSpotsRadius(vSpotsClose,:));
          vNewSpotsClose.SetColorRGBA(hex2dec('ff00ff'));
          vNewSpotsClose.SetName(sprintf('%s close to %s [%.2f]', ...
              char(vSpots.GetName), char(vSurface.GetName), vThreshold));
          vParent.AddChild(vNewSpotsClose, -1);
        end
        
        vSpotsFar = vDistances > vThreshold;
        if any(vSpotsFar)
          vNewSpotsFar = vImarisApplication.GetFactory.CreateSpots;
          vNewSpotsFar.Set(vSpotsPosition(vSpotsFar, :), ...
              vSpotsTime(vSpotsFar), zeros(sum(vSpotsFar),1));
          vNewSpotsFar.SetRadiiXYZ(vSpotsRadius(vSpotsFar,:));
          vNewSpotsFar.SetColorRGBA(hex2dec('ffff00'));
          vNewSpotsFar.SetName(sprintf('%s far from %s [%.2f]', ...
              char(vSpots.GetName), char(vSurface.GetName), vThreshold));
          vParent.AddChild(vNewSpotsFar, -1);
        end
        
        waitbar((vNumberOfSpotsSelected*(vSurfaceIndex-1)+vSpotsIndex) / ...
            (vNumberOfSpotsSelected*vNumberOfSurfacesSelected));
    end
end

close(vProgressDisplay);
